@@ -294,6 +294,7 @@ g_daemon_file_input_stream_init (GDaemonFileInputStream *info)
 {
   info->output_buffer = g_string_new ("");
   info->input_buffer = g_string_new ("");
+  info->seq_nr = 1;
 }
 
 GFileInputStream *
@@ -379,7 +380,8 @@ decode_error (GVfsDaemonSocketProtocolReply *reply, char *data, GError **error)
   g_set_error (error,
 	       g_quark_from_string (data),
 	       reply->arg1,
-	       data + strlen (data) + 1);
+               "%s",
+               data + strlen (data) + 1);
 }
 
 
@@ -514,7 +516,7 @@ iterate_read_state_machine (GDaemonFileInputStream *file, IOOperationData *io_op
 	      g_set_error (&op->ret_error,
 			   G_IO_ERROR,
 			   G_IO_ERROR_CANCELLED,
-			   _("Operation was cancelled"));
+			   "%s", _("Operation was cancelled"));
 	      return STATE_OP_DONE;
 	    }
 	  
@@ -670,6 +672,7 @@ iterate_read_state_machine (GDaemonFileInputStream *file, IOOperationData *io_op
 	      g_set_error (&op->ret_error,
 			   G_IO_ERROR,
 			   G_IO_ERROR_CANCELLED,
+                           "%s",
 			   _("Operation was cancelled"));
 	      return STATE_OP_DONE;
 	    }
@@ -740,10 +743,11 @@ g_daemon_file_input_stream_skip (GInputStream *stream,
 				 GCancellable *cancellable,
 				 GError      **error)
 {
+#if 0
   GDaemonFileInputStream *file;
 
   file = G_DAEMON_FILE_INPUT_STREAM (stream);
-  
+#endif  
   /* TODO: implement skip */
   g_assert_not_reached ();
   
@@ -777,6 +781,7 @@ iterate_close_state_machine (GDaemonFileInputStream *file, IOOperationData *io_o
 	      g_set_error (&op->ret_error,
 			   G_IO_ERROR,
 			   G_IO_ERROR_CANCELLED,
+                           "%s",
 			   _("Operation was cancelled"));
 	      return STATE_OP_DONE;
 	    }
@@ -1005,7 +1010,7 @@ iterate_seek_state_machine (GDaemonFileInputStream *file, IOOperationData *io_op
 	case SEEK_STATE_INIT:
 	  request = G_VFS_DAEMON_SOCKET_PROTOCOL_REQUEST_SEEK_SET;
 	  if (op->seek_type == G_SEEK_CUR)
-	    request = G_VFS_DAEMON_SOCKET_PROTOCOL_REQUEST_SEEK_CUR;
+	    op->offset = file->current_offset + op->offset;
 	  else if (op->seek_type == G_SEEK_END)
 	    request = G_VFS_DAEMON_SOCKET_PROTOCOL_REQUEST_SEEK_END;
 	  append_request (file, request,
@@ -1027,6 +1032,7 @@ iterate_seek_state_machine (GDaemonFileInputStream *file, IOOperationData *io_op
 	      g_set_error (&op->ret_error,
 			   G_IO_ERROR,
 			   G_IO_ERROR_CANCELLED,
+                           "%s",
 			   _("Operation was cancelled"));
 	      return STATE_OP_DONE;
 	    }
@@ -1202,7 +1208,7 @@ g_daemon_file_input_stream_seek (GFileInputStream *stream,
   if (!file->can_seek)
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-		   _("Seek not supported on stream"));
+		   "%s", _("Seek not supported on stream"));
       return FALSE;
     }
   
@@ -1232,11 +1238,7 @@ g_daemon_file_input_stream_query_info (GFileInputStream     *stream,
 				       GCancellable         *cancellable,
 				       GError              **error)
 {
-  GDaemonFileInputStream *file;
-
-  file = G_DAEMON_FILE_INPUT_STREAM (stream);
-
-  g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED, _("The query info operation is not supported"));
+  g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED, "%s", _("The query info operation is not supported"));
   
   return NULL;
 }
@@ -1497,7 +1499,6 @@ g_daemon_file_input_stream_read_async  (GInputStream        *stream,
 					gpointer            user_data)
 {
   GDaemonFileInputStream *file;
-  AsyncIterator *iterator;
   ReadOperation *op;
 
   file = G_DAEMON_FILE_INPUT_STREAM (stream);
@@ -1511,8 +1512,6 @@ g_daemon_file_input_stream_read_async  (GInputStream        *stream,
   op->buffer = buffer;
   op->buffer_size = count;
 
-  iterator = g_new0 (AsyncIterator, 1);
-  
   run_async_state_machine (file,
 			   (state_machine_iterator)iterate_read_state_machine,
 			   op,
@@ -1623,7 +1622,6 @@ g_daemon_file_input_stream_close_async (GInputStream       *stream,
 					gpointer            data)
 {
   GDaemonFileInputStream *file;
-  AsyncIterator *iterator;
   CloseOperation *op;
 
   file = G_DAEMON_FILE_INPUT_STREAM (stream);
@@ -1631,8 +1629,6 @@ g_daemon_file_input_stream_close_async (GInputStream       *stream,
   op = g_new0 (CloseOperation, 1);
   op->state = CLOSE_STATE_INIT;
 
-  iterator = g_new0 (AsyncIterator, 1);
-  
   run_async_state_machine (file,
 			   (state_machine_iterator)iterate_close_state_machine,
 			   op, io_priority,
