@@ -44,9 +44,11 @@
 #include <gvfsjobtrash.h>
 #include <gvfsjobunmount.h>
 #include <gvfsjobmountmountable.h>
+#include <gvfsjobunmountmountable.h>
 #include <gvfsjobmakedirectory.h>
 #include <gvfsjobmakesymlink.h>
 #include <gvfsjobcreatemonitor.h>
+#include <gvfsjobupload.h>
 #include <gvfsjobcopy.h>
 #include <gvfsjobmove.h>
 #include <gvfsjobsetattribute.h>
@@ -361,7 +363,71 @@ g_vfs_backend_get_mount_spec (GVfsBackend *backend)
   return backend->priv->mount_spec;
 }
 
+static void
+get_thumbnail_attributes (const char *uri,
+                          GFileInfo  *info)
+{
+  GChecksum *checksum;
+  char *filename;
+  char *basename;
 
+  checksum = g_checksum_new (G_CHECKSUM_MD5);
+  g_checksum_update (checksum, (const guchar *) uri, strlen (uri));
+
+  basename = g_strconcat (g_checksum_get_string (checksum), ".png", NULL);
+  g_checksum_free (checksum);
+
+  filename = g_build_filename (g_get_home_dir (),
+                               ".thumbnails", "normal", basename,
+                               NULL);
+
+  if (g_file_test (filename, G_FILE_TEST_IS_REGULAR))
+    g_file_info_set_attribute_byte_string (info, G_FILE_ATTRIBUTE_THUMBNAIL_PATH, filename);
+  else
+    {
+      g_free (filename);
+      filename = g_build_filename (g_get_home_dir (),
+                                   ".thumbnails", "fail",
+                                   "gnome-thumbnail-factory",
+                                   basename,
+                                   NULL);
+      
+      if (g_file_test (filename, G_FILE_TEST_IS_REGULAR))
+	g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_THUMBNAILING_FAILED, TRUE);
+    }
+  g_free (basename);
+  g_free (filename);
+}
+
+void
+g_vfs_backend_add_auto_info (GVfsBackend *backend,
+			     GFileAttributeMatcher *matcher,
+			     GFileInfo *info,
+			     const char *uri)
+{
+  GMountSpec *spec;
+  char *id;
+  
+  if (g_file_attribute_matcher_matches (matcher,
+					G_FILE_ATTRIBUTE_ID_FILESYSTEM))
+    {
+      spec = g_vfs_backend_get_mount_spec (backend);
+      if (spec)
+	{
+	  id = g_mount_spec_to_string (spec);
+	  g_file_info_set_attribute_string (info,
+					    G_FILE_ATTRIBUTE_ID_FILESYSTEM,
+					    id);
+	  g_free (id);
+	}
+    }
+
+  if (uri != NULL &&
+      g_file_attribute_matcher_matches (matcher,
+					G_FILE_ATTRIBUTE_THUMBNAIL_PATH))
+    get_thumbnail_attributes (uri, info);
+  
+}
 
 static DBusHandlerResult
 backend_dbus_handler (DBusConnection  *connection,
@@ -405,6 +471,14 @@ backend_dbus_handler (DBusConnection  *connection,
 					G_VFS_DBUS_MOUNT_INTERFACE,
 					G_VFS_DBUS_MOUNT_OP_MOUNT_MOUNTABLE))
     job = g_vfs_job_mount_mountable_new (connection, message, backend);
+  else if (dbus_message_is_method_call (message,
+					G_VFS_DBUS_MOUNT_INTERFACE,
+					G_VFS_DBUS_MOUNT_OP_UNMOUNT_MOUNTABLE))
+    job = g_vfs_job_unmount_mountable_new (connection, message, backend, FALSE);
+  else if (dbus_message_is_method_call (message,
+					G_VFS_DBUS_MOUNT_INTERFACE,
+					G_VFS_DBUS_MOUNT_OP_EJECT_MOUNTABLE))
+    job = g_vfs_job_unmount_mountable_new (connection, message, backend, TRUE);
   else if (dbus_message_is_method_call (message,
 					G_VFS_DBUS_MOUNT_INTERFACE,
 					G_VFS_DBUS_MOUNT_OP_SET_DISPLAY_NAME))
@@ -429,6 +503,10 @@ backend_dbus_handler (DBusConnection  *connection,
 					G_VFS_DBUS_MOUNT_INTERFACE,
 					G_VFS_DBUS_MOUNT_OP_COPY))
     job = g_vfs_job_copy_new (connection, message, backend);
+  else if (dbus_message_is_method_call (message,
+					G_VFS_DBUS_MOUNT_INTERFACE,
+					G_VFS_DBUS_MOUNT_OP_UPLOAD))
+    job = g_vfs_job_upload_new (connection, message, backend);
   else if (dbus_message_is_method_call (message,
 					G_VFS_DBUS_MOUNT_INTERFACE,
 					G_VFS_DBUS_MOUNT_OP_MOVE))
@@ -481,7 +559,8 @@ g_vfs_backend_register_mount (GVfsBackend *backend,
   if (message == NULL)
     _g_dbus_oom ();
 
-  if (backend->priv->stable_name != NULL)
+  if (backend->priv->stable_name != NULL &&
+      *backend->priv->stable_name != 0)
    stable_name = backend->priv->stable_name;
   else
    stable_name = backend->priv->display_name;
