@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* GIO - GLib Input, Output and Streaming Library
  * 
  * Copyright (C) 2008 Benjamin Otte <otte@gnome.org>
@@ -45,6 +44,8 @@
 #include "gvfsdaemonutils.h"
 #include "gvfskeyring.h"
 
+#define MOUNT_ICON_NAME "drive-removable-media"
+
 /* #define PRINT_DEBUG  */
 
 #ifdef PRINT_DEBUG
@@ -291,11 +292,12 @@ archive_file_get_from_path (ArchiveFile *file, const char *filename, gboolean ad
 #define archive_file_find(ba, filename) archive_file_get_from_path((ba)->files, (filename) + 1, FALSE)
 
 static void
-create_root_file (GVfsBackendArchive *ba, GIcon *icon)
+create_root_file (GVfsBackendArchive *ba)
 {
   ArchiveFile *root = g_slice_new0 (ArchiveFile);
   GFileInfo *info;
   char *s, *display_name;
+  GIcon *icon;
 
   root = g_slice_new0 (ArchiveFile);
   root->name = g_strdup ("/");
@@ -319,7 +321,16 @@ create_root_file (GVfsBackendArchive *ba, GIcon *icon)
   g_file_info_set_content_type (info, "inode/directory");
   g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE, "inode/directory");
 
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ, TRUE);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE, FALSE);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_DELETE, FALSE);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE, TRUE);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_TRASH, FALSE);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_RENAME, FALSE);
+
+  icon = g_themed_icon_new ("folder");
   g_file_info_set_icon (info, icon);
+  g_object_unref (icon);
 }
 
 static void
@@ -335,19 +346,19 @@ archive_file_set_info_from_entry (ArchiveFile *	        file,
   g_file_info_set_attribute_uint64 (info,
 				    G_FILE_ATTRIBUTE_TIME_ACCESS,
 				    archive_entry_atime (entry));
-  g_file_info_set_attribute_uint64 (info,
+  g_file_info_set_attribute_uint32 (info,
 				    G_FILE_ATTRIBUTE_TIME_ACCESS_USEC,
 				    archive_entry_atime_nsec (entry) / 1000);
   g_file_info_set_attribute_uint64 (info,
 				    G_FILE_ATTRIBUTE_TIME_CHANGED,
 				    archive_entry_ctime (entry));
-  g_file_info_set_attribute_uint64 (info,
+  g_file_info_set_attribute_uint32 (info,
 				    G_FILE_ATTRIBUTE_TIME_CHANGED_USEC,
 				    archive_entry_ctime_nsec (entry) / 1000);
   g_file_info_set_attribute_uint64 (info,
 				    G_FILE_ATTRIBUTE_TIME_MODIFIED,
 				    archive_entry_mtime (entry));
-  g_file_info_set_attribute_uint64 (info,
+  g_file_info_set_attribute_uint32 (info,
 				    G_FILE_ATTRIBUTE_TIME_MODIFIED_USEC,
 				    archive_entry_mtime_nsec (entry) / 1000);
 
@@ -382,6 +393,13 @@ archive_file_set_info_from_entry (ArchiveFile *	        file,
   g_file_info_set_size (info,
 			archive_entry_size (entry));
 
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ, TRUE);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE, FALSE);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_DELETE, FALSE);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE, type == G_FILE_TYPE_DIRECTORY);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_TRASH, FALSE);
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_RENAME, FALSE);
+
   /* FIXME: add info for these
 dev_t			 archive_entry_dev(struct archive_entry *);
 dev_t			 archive_entry_devmajor(struct archive_entry *);
@@ -439,12 +457,20 @@ create_file_tree (GVfsBackendArchive *ba, GVfsJob *job)
   do
     {
       result = archive_read_next_header (archive->archive, &entry);
-      if (result == ARCHIVE_OK)
+      if (result >= ARCHIVE_WARN && result <= ARCHIVE_OK)
 	{
+  	  if (result < ARCHIVE_OK) {
+  	    DEBUG ("archive_read_next_header: result = %d, error = '%s'\n", result, archive_error_string (archive->archive));
+  	    archive_set_error (archive->archive, ARCHIVE_OK, "No error");
+  	    archive_clear_error (archive->archive);
+	  }
+  
 	  ArchiveFile *file = archive_file_get_from_path (ba->files, 
 	                                                  archive_entry_pathname (entry), 
 							  TRUE);
-	  archive_file_set_info_from_entry (file, entry);
+          /* Don't set info for root */
+          if (file != ba->files)
+            archive_file_set_info_from_entry (file, entry);
 	  archive_read_data_skip (archive->archive);
 	}
     }
@@ -501,11 +527,11 @@ do_mount (GVfsBackend *backend,
           return;
         }
       
-      archive->file = g_file_new_for_uri (filename);
+      archive->file = g_file_new_for_commandline_arg (filename);
       g_free (filename);
     }
   else
-    archive->file = g_file_new_for_uri (file);
+    archive->file = g_file_new_for_commandline_arg (file);
   
   DEBUG ("Trying to mount %s\n", g_file_get_uri (archive->file));
 
@@ -544,15 +570,10 @@ do_mount (GVfsBackend *backend,
   g_vfs_backend_set_display_name (backend, g_file_info_get_display_name (info));
 
   icon = g_file_info_get_icon (info);
-#if 0
-  if (G_IS_THEMED_ICON (icon))
-    g_vfs_backend_set_icon_name (backend, 
-	                         g_themed_icon_get_names (G_THEMED_ICON (icon))[0]);
-  else
-#endif
-    g_vfs_backend_set_icon_name (backend, "package-x-generic");
 
-  create_root_file (archive, icon);
+  g_vfs_backend_set_icon_name (backend, MOUNT_ICON_NAME);
+
+  create_root_file (archive);
   create_file_tree (archive, G_VFS_JOB (job));
   g_object_unref (info);
 }
@@ -585,8 +606,14 @@ do_open_for_read (GVfsBackend *       backend,
   do
     {
       result = archive_read_next_header (archive->archive, &entry);
-      if (result == ARCHIVE_OK)
+      if (result >= ARCHIVE_WARN && result <= ARCHIVE_OK)
         {
+	  if (result < ARCHIVE_OK) {
+	    DEBUG ("do_open_for_read: result = %d, error = '%s'\n", result, archive_error_string (archive->archive));
+	    archive_set_error (archive->archive, ARCHIVE_OK, "No error");
+	    archive_clear_error (archive->archive);
+	  }
+                              
           if (g_str_equal (archive_entry_pathname (entry), filename + 1))
             {
               /* SUCCESS */
@@ -712,6 +739,19 @@ do_enumerate (GVfsBackend *backend,
   g_vfs_job_succeeded (G_VFS_JOB (job));
 }
 
+static gboolean
+try_query_fs_info (GVfsBackend *backend,
+                   GVfsJobQueryFsInfo *job,
+                   const char *filename,
+                   GFileInfo *info,
+                   GFileAttributeMatcher *attribute_matcher)
+{
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_READONLY, TRUE);
+  g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_FILESYSTEM_USE_PREVIEW, G_FILESYSTEM_PREVIEW_TYPE_IF_LOCAL);
+  g_vfs_job_succeeded (G_VFS_JOB (job));
+  return TRUE;
+}
+
 static void
 g_vfs_backend_archive_class_init (GVfsBackendArchiveClass *klass)
 {
@@ -727,4 +767,5 @@ g_vfs_backend_archive_class_init (GVfsBackendArchiveClass *klass)
   backend_class->read = do_read;
   backend_class->enumerate = do_enumerate;
   backend_class->query_info = do_query_info;
+  backend_class->try_query_fs_info = try_query_fs_info;
 }
