@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* GIO - GLib Input, Output and Streaming Library
  * 
  * Copyright (C) 2006-2007 Red Hat, Inc.
@@ -573,6 +572,10 @@ list_trash_dirs (void)
         has_trash_files = TRUE;
     }
 
+  g_list_foreach (topdirs, (GFunc) g_free, NULL);
+  g_list_free (topdirs);
+  g_list_free (topdirs_info);
+
   return g_list_reverse (dirs);
 }
 
@@ -671,7 +674,6 @@ do_open_for_read (GVfsBackend *backend,
     }
 }
 
-
 static void
 do_read (GVfsBackend *backend,
          GVfsJobRead *job,
@@ -752,7 +754,6 @@ do_close_read (GVfsBackend *backend,
       g_vfs_job_failed_from_error (G_VFS_JOB (job), error);
       g_error_free (error);
     }
-
 }
 
 typedef struct {
@@ -951,9 +952,14 @@ add_extra_trash_info (GFileInfo *file_info,
 
           if (orig_path_unescaped)
             {
-              /* Set display name and edit name based of original basename */
-              display_name = g_filename_display_basename (orig_path_unescaped);
+              if (g_path_is_absolute (orig_path_unescaped))
+                orig_path = g_build_filename (orig_path_unescaped, relative_path, NULL);
+              else
+                orig_path = g_build_filename (topdir, orig_path_unescaped, relative_path, NULL);
+              g_free (orig_path_unescaped);
 
+              /* Set display name and edit name based of original basename */
+              display_name = g_filename_display_basename (orig_path);
               g_file_info_set_edit_name (file_info, display_name);
               
               if (strstr (display_name, "\357\277\275") != NULL)
@@ -975,22 +981,11 @@ add_extra_trash_info (GFileInfo *file_info,
               
               g_free (display_name);
               
-              
-              /* Set orig_path */
-              
-              if (g_path_is_absolute (orig_path_unescaped))
-                orig_path = g_build_filename (orig_path_unescaped, relative_path, NULL);
-              else
-                orig_path = g_build_filename (topdir, orig_path_unescaped, relative_path, NULL);
-
-              
               g_file_info_set_attribute_byte_string (file_info,
                                                      "trash::orig-path",
                                                      orig_path);
               g_free (orig_path);
-              g_free (orig_path_unescaped);
             }
-          
 
           g_free (orig_path_key);
         }
@@ -1015,9 +1010,6 @@ enumerate_root_trashdir (GVfsBackend *backend,
 { 
   GFile *file, *files_file;
   GFileEnumerator *enumerator;
-  GFileInfo *info;
-  const char *name;
-  char *new_name, *new_name_escaped;
   char *info_dir;
 
   info_dir = g_build_filename (trashdir, "info", NULL);
@@ -1035,10 +1027,15 @@ enumerate_root_trashdir (GVfsBackend *backend,
 
   if (enumerator)
     {
+      GFileInfo *info;
+
       while ((info = g_file_enumerator_next_file (enumerator,
                                                   job ? G_VFS_JOB (job)->cancellable : NULL,
                                                   NULL)) != NULL)
         {
+          const char *name;
+          char *new_name, *new_name_escaped;
+
           name = g_file_info_get_name (info);
 
           /* Get the display name, etc */
@@ -1048,7 +1045,6 @@ enumerate_root_trashdir (GVfsBackend *backend,
                                 name,
                                 NULL);
 
-          
           /* Update the name to also have the trash dir */
           new_name = g_build_filename (trashdir, name, NULL);
           new_name_escaped = escape_pathname (new_name);
@@ -1067,6 +1063,8 @@ enumerate_root_trashdir (GVfsBackend *backend,
                                NULL);
       g_object_unref (enumerator);
     }
+
+  g_free (info_dir);
 }
 
 static GList *
